@@ -50,6 +50,7 @@ namespace PyXRootD
       { "EnvPutInt_cpp",        EnvPutInt_cpp,        METH_VARARGS, "Puts an int into XrdCl environment." },
       { "EnvGetInt_cpp",        EnvGetInt_cpp,        METH_VARARGS, "Gets an int from XrdCl environment." },
       { "XrdVersion_cpp",       XrdVersion_cpp,       METH_VARARGS, "Get the XRootD client version." },
+      { "EnvGetDefault_cpp",    EnvGetDefault_cpp,    METH_VARARGS, "Get default values from XrdCl environment" },
       { NULL, NULL, 0, NULL }
     };
 
@@ -79,10 +80,12 @@ namespace PyXRootD
   PyMODINIT_FUNC initclient( void )
 #endif
   {
+#if PY_VERSION_HEX < 0x03070000
     // Ensure GIL state is initialized
     if ( !PyEval_ThreadsInitialized() ) {
       PyEval_InitThreads();
     }
+#endif
 
     FileSystemType.tp_new = PyType_GenericNew;
     if ( PyType_Ready( &FileSystemType ) < 0 ) {
