@@ -42,6 +42,7 @@
 #include "XrdAcc/XrdAccGroups.hh"
 #include "XrdNet/XrdNetAddrInfo.hh"
 #include "XrdOuc/XrdOucUtils.hh"
+#include "XrdSec/XrdSecEntityAttr.hh"
 #include "XrdSys/XrdSysPlugin.hh"
   
 /******************************************************************************/
@@ -122,7 +123,13 @@ XrdAccPrivs XrdAccAccess::Access(const XrdSecEntity    *Entity,
 
 // Setup the id (we don't need a lock for that)
 //
-   if (Entity->name)
+   std::string username;
+   auto got_token = Entity->eaAPI->Get("request.name", username);
+   if (got_token && !username.empty())
+      {eInfo.name = username.c_str();
+       isuser = true;
+      }
+   else if (Entity->name)
       {eInfo.name = Entity->name;
        isuser = (*eInfo.name != 0);
       } else {
@@ -293,11 +300,18 @@ int XrdAccAccess::Audit(const int              accok,
                                     "read",            // 8
                                     "readdir",         // 9
                                     "rename",          // 10
-                                    "stat",            // 10
-                                    "update"           // 12
+                                    "stat",            // 11
+                                    "update",          // 12
+                                    "excl_create",     // 13
+                                    "excl_insert"      // 14
                              };
    const char *opname = (oper > AOP_LastOp ? "???" : Opername[oper]);
-   const char *id   = (Entity->name ? (const char *)Entity->name : "*");
+   std::string username;
+   const char *id = "*";
+   auto got_token = Entity->eaAPI->Get("request.name", username);
+   if (got_token && !username.empty()) {
+       id = username.c_str();
+   } else if (Entity->name) id = Entity->name;
    const char *host = (Entity->host ? (const char *)Entity->host : "?");
    char atype[XrdSecPROTOIDSIZE+1];
 
@@ -415,8 +429,12 @@ int XrdAccAccess::Test(const XrdAccPrivs priv,const Access_Operation oper)
                                 XrdAccPriv_Readdir,              // 9
                                 XrdAccPriv_Rename,               // 10
                                 XrdAccPriv_Lookup,               // 11
-                                XrdAccPriv_Update                // 12
+                                XrdAccPriv_Update,               // 12
+                                (XrdAccPrivs)0xffff,             // 13
+                                (XrdAccPrivs)0xffff              // 14
                                };
+   // Note AOP_Excl* does not have a corresponding XrdAccPrivs; this is on
+   // purpose as the Excl* privilege is not modelled within the AuditDB framework.
    if (oper < 0 || oper > AOP_LastOp) return 0;
    return (int)(need[oper] & priv) == need[oper];
 }
