@@ -20,6 +20,9 @@
 #include "XrdCl/XrdClInQueue.hh"
 #include "XrdCl/XrdClPostMasterInterfaces.hh"
 #include "XrdCl/XrdClMessage.hh"
+#include "XrdCl/XrdClLog.hh"
+#include "XrdCl/XrdClDefaultEnv.hh"
+#include "XrdCl/XrdClConstants.hh"
 
 #include <arpa/inet.h>              // for network unmarshalling stuff
 
@@ -35,67 +38,16 @@ namespace XrdCl
 
     ServerResponse *rsp = (ServerResponse *)msg.GetBuffer();
 
-    // We got an async message
+    // We only care about async responses, but those are extracted now
+    // in the SocketHandler
     if( rsp->hdr.status == kXR_attn )
-    {
-      if( msg.GetSize() < 12 )
-        return true;
-
-      // We only care about async responses
-      if( rsp->body.attn.actnum != (int32_t)htonl(kXR_asynresp) )
-        return true;
-
-      if( msg.GetSize() < 24 )
-        return true;
-
-      ServerResponse *embRsp = (ServerResponse*)msg.GetBuffer(16);
-      sid = ((uint16_t)embRsp->hdr.streamid[1] << 8) | (uint16_t)embRsp->hdr.streamid[0];
-    }
+      return true;
     else
-    {
       sid = ((uint16_t)rsp->hdr.streamid[1] << 8) | (uint16_t)rsp->hdr.streamid[0];
-    }
 
     return false;
   }
 
-  //----------------------------------------------------------------------------
-  // Add a message to the queue
-  //----------------------------------------------------------------------------
-  bool InQueue::AddMessage( std::shared_ptr<Message> msg )
-  {
-    uint16_t            action  = 0;
-    MsgHandler* handler = 0;
-    uint16_t msgSid = 0;
-
-    if (DiscardMessage(*msg, msgSid))
-    {
-      return true;
-    }
-
-    // Lookup the sid in the map of handlers
-    pMutex.Lock();
-    HandlerMap::iterator it = pHandlers.find(msgSid);
-
-    if (it != pHandlers.end())
-    {
-      handler = it->second.first;
-      action  = handler->Examine( msg );
-
-      if( action & MsgHandler::RemoveHandler )
-        pHandlers.erase( it );
-    }
-    else
-      pMessages[msgSid] = msg;
-
-    pMutex.UnLock();
-
-    if( handler && !(action & MsgHandler::NoProcess) )
-      handler->Process();
-
-    return true;
-  }
-
   //----------------------------------------------------------------------------
   // Add a listener that should be notified about incoming messages
   //----------------------------------------------------------------------------
@@ -103,15 +55,6 @@ namespace XrdCl
   {
     uint16_t handlerSid = handler->GetSid();
     XrdSysMutexHelper scopedLock( pMutex );
-    MessageMap::iterator it = pMessages.find(handlerSid);
-
-    //--------------------------------------------------------------------------
-    // If there is a leftover message in the in-queue simply remove it, there's
-    // no way this is the actual message we are waiting for as we just send it
-    // over the wire and we are still occupying the event-loop thread
-    //--------------------------------------------------------------------------
-    if( ( rmMsg = it != pMessages.end() ) )
-      pMessages.erase( it );
 
     pHandlers[handlerSid] = HandlerAndExpire( handler, expires );
   }
@@ -121,8 +64,8 @@ namespace XrdCl
   // is stored in msg
   //----------------------------------------------------------------------------
   MsgHandler *InQueue::GetHandlerForMessage( std::shared_ptr<Message> &msg,
-						                                         time_t                   &expires,
-						                                         uint16_t                 &action )
+						                                 time_t                   &expires,
+						                                 uint16_t                 &action )
   {
     time_t   exp = 0;
     uint16_t act = 0;
@@ -139,12 +82,20 @@ namespace XrdCl
 
     if (it != pHandlers.end())
     {
+      Log *log = DefaultEnv::GetLog();
       handler = it->second.first;
       act     = handler->Examine( msg );
       exp     = it->second.second;
+      log->Debug( ExDbgMsg, "[msg: 0x%x] Assigned MsgHandler: 0x%x.",
+                  msg.get(), handler );
+
 
       if( act & MsgHandler::RemoveHandler )
+      {
         pHandlers.erase( it );
+        log->Debug( ExDbgMsg, "[handler: 0x%x] Removed MsgHandler: 0x%x from the in-queue.",
+                    handler, handler );
+      }
     }
 
     if( handler )
@@ -175,6 +126,10 @@ namespace XrdCl
     uint16_t handlerSid = handler->GetSid();
     XrdSysMutexHelper scopedLock( pMutex );
     pHandlers.erase(handlerSid);
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( ExDbgMsg, "[handler: 0x%x] Removed MsgHandler: 0x%x from the in-queue.",
+                handler, handler );
+
   }
 
   //----------------------------------------------------------------------------
