@@ -44,6 +44,11 @@
 
 namespace XrdCl
 {
+  //----------------------------------------------------------------------------
+  // Statics
+  //----------------------------------------------------------------------------
+  RAtomic_uint64_t        Stream::sSessCntGen{0};
+
   //----------------------------------------------------------------------------
   // Incoming message helper
   //----------------------------------------------------------------------------
@@ -342,11 +347,14 @@ namespace XrdCl
   void Stream::ForceConnect()
   {
     XrdSysMutexHelper scopedLock( pMutex );
-    pSubStreams[0]->status = Socket::Disconnected;
-    XrdCl::PathID path( 0, 0 );
-    XrdCl::XRootDStatus st = EnableLink( path );
-    if( !st.IsOK() )
-      OnConnectError( 0, st );
+    if( pSubStreams[0]->status == Socket::Connecting )
+    {
+      pSubStreams[0]->status = Socket::Disconnected;
+      XrdCl::PathID path( 0, 0 );
+      XrdCl::XRootDStatus st = EnableLink( path );
+      if( !st.IsOK() )
+        OnConnectError( 0, st );
+    }
   }
 
   //----------------------------------------------------------------------------
@@ -394,11 +402,11 @@ namespace
       //------------------------------------------------------------------------
       // Constructor
       //------------------------------------------------------------------------
-      StreamConnectorTask( XrdCl::Stream *stream ):
-        pStream( stream )
+      StreamConnectorTask( const XrdCl::URL &url, const std::string &n ):
+        url( url )
       {
         std::string name = "StreamConnectorTask for ";
-        name += stream->GetName();
+        name += n;
         SetName( name );
       }
 
@@ -407,12 +415,12 @@ namespace
       //------------------------------------------------------------------------
       time_t Run( time_t )
       {
-        pStream->ForceConnect();
+        XrdCl::DefaultEnv::GetPostMaster()->ForceReconnect( url );
         return 0;
       }
 
     private:
-      XrdCl::Stream *pStream;
+      XrdCl::URL url;
   };
 }
 
@@ -615,7 +623,7 @@ namespace XrdCl
       pLastFatalError  = XRootDStatus();
       pConnectionCount = 0;
       uint16_t numSub = pTransport->SubStreamNumber( *pChannelData );
-      ++pSessionId;
+      pSessionId = ++sSessCntGen;
 
       //------------------------------------------------------------------------
       // Create the streams if they don't exist yet
@@ -770,7 +778,7 @@ namespace XrdCl
       log->Info( PostMasterMsg, "[%s] Attempting reconnection in %d "
                  "seconds.", pStreamName.c_str(), pConnectionWindow-elapsed );
 
-      Task *task = new ::StreamConnectorTask( this );
+      Task *task = new ::StreamConnectorTask( *pUrl, pStreamName );
       pTaskManager->RegisterTask( task, pConnectionInitTime+pConnectionWindow );
       return;
     }
@@ -1016,13 +1024,13 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Call back when a message has been reconstructed
   //----------------------------------------------------------------------------
-  void Stream::OnReadTimeout( uint16_t substream )
+  bool Stream::OnReadTimeout( uint16_t substream )
   {
     //--------------------------------------------------------------------------
     // We only take the main stream into account
     //--------------------------------------------------------------------------
     if( substream != 0 )
-      return;
+      return true;
 
     //--------------------------------------------------------------------------
     // Check if there is no outgoing messages and if the stream TTL is elapesed.
@@ -1062,7 +1070,7 @@ namespace XrdCl
         // object that aggregates this Stream.
         //----------------------------------------------------------------------
         DefaultEnv::GetPostMaster()->ForceDisconnect( *pUrl );
-        return;
+        return false;
       }
     }
 
@@ -1075,14 +1083,17 @@ namespace XrdCl
     {
       scopedLock.UnLock();
       OnError( substream, st );
+      return false;
     }
+    return true;
   }
 
   //----------------------------------------------------------------------------
   // Call back when a message has been reconstru
   //----------------------------------------------------------------------------
-  void Stream::OnWriteTimeout( uint16_t /*substream*/ )
+  bool Stream::OnWriteTimeout( uint16_t /*substream*/ )
   {
+    return true;
   }
 
   //----------------------------------------------------------------------------
