@@ -287,9 +287,12 @@ namespace XrdCl
     else
       pPath = path;
 
-    std::string::iterator back = pPath.end() - 1;
-    if( pProtocol == "file" && *back == '/' )
-      pPath.erase( back );
+    if( !pPath.empty() )
+    {
+      std::string::iterator back = pPath.end() - 1;
+      if( pProtocol == "file" && *back == '/' )
+        pPath.erase( back );
+    }
 
     ComputeURL();
     return true;
@@ -490,7 +493,7 @@ namespace XrdCl
   //------------------------------------------------------------------------
   std::string URL::GetChannelId() const
   {
-    std::string ret = pProtocol + "://" + pHostId;
+    std::string ret = pProtocol + "://" + pHostId + "/";
     bool hascgi = false;
 
     std::string keys[] = { "xrdcl.intent",
