@@ -28,15 +28,16 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
   
-#include <ctype.h>
+#include <cctype>
 #include <netdb.h>
-#include <stdio.h>
+#include <cstdio>
 #include <unistd.h>
 #include <arpa/inet.h>
 #include <sys/types.h>
 
 #include "XrdNet/XrdNetAddr.hh"
 #include "XrdNet/XrdNetCache.hh"
+#include "XrdNet/XrdNetIdentity.hh"
 #include "XrdNet/XrdNetUtils.hh"
 #include "XrdSys/XrdSysE2T.hh"
 
@@ -90,15 +91,15 @@ bool               XrdNetAddr::dynDNS       = false;
 /******************************************************************************/
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
-  
+
 XrdNetAddr::XrdNetAddr(int port) : XrdNetAddrInfo()
 {
-   char buff[1024];
+   const char *fqn = XrdNetIdentity::FQN();
 
-// Get our host name and initialize this object with it
+// Otherwise, we cannot initialize this object so force an error!
 //
-   gethostname(buff, sizeof(buff));
-   Set(buff, port);
+   if (!fqn) fqn = "No_DNS_Name!";
+   Set(fqn, port);
 }
 
 /******************************************************************************/
@@ -173,7 +174,7 @@ bool XrdNetAddr::Register(const char *hName)
    XrdNetAddr *aListVec = 0;
    int i, aListNum;
 
-// Step one is to make sure the incomming name is not an address
+// Step one is to make sure the incoming name is not an address
 //
    if (!isHostName(hName)) return false;
 
