@@ -409,7 +409,7 @@ int XrdNetIF::GetDest(char *dest, int dlen, ifType ifT, bool prefn)
 /*                                 G e t I F                                  */
 /******************************************************************************/
 
-#define prtaddr(x) cerr <<"Addr!!! " << *x <<endl;
+#define prtaddr(x) std::cerr <<"Addr!!! " << *x <<std::endl;
 
 int XrdNetIF::GetIF(XrdOucTList **ifList, const char **eText)
 {
