@@ -431,8 +431,8 @@ uint32_t crc32c_sw(uint32_t crc, void const *buf, size_t len) {
 
 #ifdef TEST
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include "load.h"
 
 int main(int argc, char **argv) {
