@@ -1171,8 +1171,10 @@ void File::ProcessDirectReadFinished(ReadRequest *rreq, int bytes_read, int erro
 
    if (error_cond)
       rreq->update_error_cond(error_cond);
-   else
+   else {
       rreq->m_stats.m_BytesBypassed += bytes_read;
+      rreq->m_bytes_read += bytes_read;
+   }
 
    rreq->m_direct_done = true;
 
@@ -1495,7 +1497,7 @@ void File::insert_remote_location(const std::string &loc)
    if ( ! loc.empty())
    {
       size_t p = loc.find_first_of('@');
-      m_remote_locations.insert(&loc[p != string::npos ? p + 1 : 0]);
+      m_remote_locations.insert(&loc[p != std::string::npos ? p + 1 : 0]);
    }
 }
 
