@@ -28,6 +28,10 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
+#if defined(__clang__) && defined(_FORTIFY_SOURCE)
+#undef _FORTIFY_SOURCE
+#endif
+
 #include <sys/types.h>
 #include <cstdarg>
 #include <unistd.h>
@@ -41,6 +45,22 @@
 /******************************************************************************/
 
 #include "XrdPosix/XrdPosixExtern.hh"
+
+#ifdef MUSL
+#undef creat64
+#undef fseeko64
+#undef ftello64
+#undef ftruncate64
+#undef lseek64
+#undef open64
+#undef pread64
+#undef pwrite64
+#undef readdir64
+#undef readdir64_r
+#undef statfs64
+#undef statvfs64
+#undef truncate64
+#endif
  
 /******************************************************************************/
 /*                   G l o b a l   D e c l a r a t i o n s                    */
