@@ -192,10 +192,10 @@ int XrdSutGetLine(XrdOucString &line, const char *prompt)
    
    // Print prompt, if requested
    if (prompt)
-      cout << prompt;
+      std::cout << prompt;
 
    // Get line
-   cin.getline(bin,XrdSutMAXBUF-1);
+   std::cin.getline(bin,XrdSutMAXBUF-1);
 
    // Fill input
    line = bin;
@@ -217,7 +217,7 @@ bool XrdSutAskConfirm(const char *msg1, bool defact, const char *msg2)
    bool rc = defact;
 
    if (msg2)
-      cout << msg2;
+      std::cout << msg2;
    XrdOucString ask;
    XrdOucString prompt = defact ? " [y]: " : " [n]: ";
    if (msg1)
